"\nThe API basically only provides one class. You can create a :class:`Script` and\nuse its methods.\n\nAdditionally you can add a debug function with :func:`set_debug_function`.\nAlternatively, if you don't need a custom function and are happy with printing\ndebug messages to stdout, simply call :func:`set_debug_function` without\narguments.\n"
_H='classdef'
_G='funcdef'
_F='keyword'
_E='operator'
_D='__main__'
_C=True
_B=False
_A=None
import sys
from pathlib import Path
import parso
from parso.python import tree
from jedi.parser_utils import get_executable_nodes
from jedi import debug
from jedi import settings
from jedi import cache
from jedi.file_io import KnownContentFileIO
from jedi.api import classes
from jedi.api import interpreter
from jedi.api import helpers
from jedi.api.helpers import validate_line_column
from jedi.api.completion import Completion,search_in_module
from jedi.api.keywords import KeywordName
from jedi.api.environment import InterpreterEnvironment
from jedi.api.project import get_default_project,Project
from jedi.api.errors import parso_to_jedi_errors
from jedi.api import refactoring
from jedi.api.refactoring.extract import extract_function,extract_variable
from jedi.inference import InferenceState
from jedi.inference import imports
from jedi.inference.references import find_references
from jedi.inference.arguments import try_iter_content
from jedi.inference.helpers import infer_call_of_leaf
from jedi.inference.sys_path import transform_path_to_dotted
from jedi.inference.syntax_tree import tree_name_to_values
from jedi.inference.value import ModuleValue
from jedi.inference.base_value import ValueSet
from jedi.inference.value.iterable import unpack_tuple_to_dict
from jedi.inference.gradual.conversion import convert_names,convert_values
from jedi.inference.gradual.utils import load_proper_stub_module
from jedi.inference.utils import to_list
sys.setrecursionlimit(3000)
class Script:
	"\n    A Script is the base for completions, goto or whatever you want to do with\n    Jedi. The counter part of this class is :class:`Interpreter`, which works\n    with actual dictionaries and can work with a REPL. This class\n    should be used when a user edits code in an editor.\n\n    You can either use the ``code`` parameter or ``path`` to read a file.\n    Usually you're going to want to use both of them (in an editor).\n\n    The Script's ``sys.path`` is very customizable:\n\n    - If `project` is provided with a ``sys_path``, that is going to be used.\n    - If `environment` is provided, its ``sys.path`` will be used\n      (see :func:`Environment.get_sys_path <jedi.api.environment.Environment.get_sys_path>`);\n    - Otherwise ``sys.path`` will match that of the default environment of\n      Jedi, which typically matches the sys path that was used at the time\n      when Jedi was imported.\n\n    Most methods have a ``line`` and a ``column`` parameter. Lines in Jedi are\n    always 1-based and columns are always zero based. To avoid repetition they\n    are not always documented. You can omit both line and column. Jedi will\n    then just do whatever action you are calling at the end of the file. If you\n    provide only the line, just will complete at the end of that line.\n\n    .. warning:: By default :attr:`jedi.settings.fast_parser` is enabled, which means\n        that parso reuses modules (i.e. they are not immutable). With this setting\n        Jedi is **not thread safe** and it is also not safe to use multiple\n        :class:`.Script` instances and its definitions at the same time.\n\n        If you are a normal plugin developer this should not be an issue. It is\n        an issue for people that do more complex stuff with Jedi.\n\n        This is purely a performance optimization and works pretty well for all\n        typical usages, however consider to turn the setting off if it causes\n        you problems. See also\n        `this discussion <https://github.com/davidhalter/jedi/issues/1240>`_.\n\n    :param code: The source code of the current file, separated by newlines.\n    :type code: str\n    :param path: The path of the file in the file system, or ``''`` if\n        it hasn't been saved yet.\n    :type path: str or pathlib.Path or None\n    :param Environment environment: Provide a predefined :ref:`Environment <environments>`\n        to work with a specific Python version or virtualenv.\n    :param Project project: Provide a :class:`.Project` to make sure finding\n        references works well, because the right folder is searched. There are\n        also ways to modify the sys path and other things.\n    "
	def __init__(A,code=_A,*,path=_A,environment=_A,project=_A):
		D=project;C=code;B=path;A._orig_path=B
		if isinstance(B,str):B=Path(B)
		A.path=B.absolute()if B else _A
		if C is _A:
			if B is _A:raise ValueError('Must provide at least one of code or path')
			with open(B,'rb')as E:C=E.read()
		if D is _A:D=get_default_project(_A if A.path is _A else A.path.parent)
		A._inference_state=InferenceState(D,environment=environment,script_path=A.path);debug.speed('init');A._module_node,C=A._inference_state.parse_and_get_code(code=C,path=A.path,use_latest_grammar=B and B.suffix=='.pyi',cache=_B,diff_cache=settings.fast_parser,cache_path=settings.cache_directory);debug.speed('parsed');A._code_lines=parso.split_lines(C,keepends=_C);A._code=C;cache.clear_time_caches();debug.reset_time()
	@cache.memoize_method
	def _get_module(self):
		A=self;B=_A;D=_B
		if A.path is not _A:
			E,H=transform_path_to_dotted(A._inference_state.get_sys_path(add_parent_paths=_B),A.path)
			if E is not _A:B=E;D=H
		if A.path is _A:C=_A
		else:C=KnownContentFileIO(A.path,A._code)
		if A.path is not _A and A.path.suffix=='.pyi':
			F=load_proper_stub_module(A._inference_state,A._inference_state.latest_grammar,C,B,A._module_node)
			if F is not _A:return F
		if B is _A:B=_D,
		G=ModuleValue(A._inference_state,A._module_node,file_io=C,string_names=B,code_lines=A._code_lines,is_package=D)
		if B[0]not in('builtins','typing'):A._inference_state.module_cache.add(B,ValueSet([G]))
		return G
	def _get_module_context(A):return A._get_module().as_context()
	def __repr__(A):return'<%s: %s %r>'%(A.__class__.__name__,repr(A._orig_path),A._inference_state.environment)
	@validate_line_column
	def complete(self,line=_A,column=_A,*,fuzzy=_B):
		'\n        Completes objects under the cursor.\n\n        Those objects contain information about the completions, more than just\n        names.\n\n        :param fuzzy: Default False. Will return fuzzy completions, which means\n            that e.g. ``ooa`` will match ``foobar``.\n        :return: Completion objects, sorted by name. Normal names appear\n            before "private" names that start with ``_`` and those appear\n            before magic methods and name mangled names that start with ``__``.\n        :rtype: list of :class:`.Completion`\n        ';A=self;A._inference_state.reset_recursion_limitations()
		with debug.increase_indent_cm('complete'):B=Completion(A._inference_state,A._get_module_context(),A._code_lines,(line,column),A.get_signatures,fuzzy=fuzzy);return B.complete()
	@validate_line_column
	def infer(self,line=_A,column=_A,*,only_stubs=_B,prefer_stubs=_B):
		G='string';E=column;B=self;B._inference_state.reset_recursion_limitations();"\n        Return the definitions of under the cursor. It is basically a wrapper\n        around Jedi's type inference.\n\n        This method follows complicated paths and returns the end, not the\n        first definition. The big difference between :meth:`goto` and\n        :meth:`infer` is that :meth:`goto` doesn't\n        follow imports and statements. Multiple objects may be returned,\n        because depending on an option you can have two different versions of a\n        function.\n\n        :param only_stubs: Only return stubs for this method.\n        :param prefer_stubs: Prefer stubs to Python objects for this method.\n        :rtype: list of :class:`.Name`\n        ";F=line,E;A=B._module_node.get_name_of_position(F)
		if A is _A:
			A=B._module_node.get_leaf_for_position(F)
			if A is _A or A.type==G:return[]
			if A.end_pos==(line,E)and A.type==_E:
				C=A.get_next_leaf()
				if C.start_pos==A.end_pos and C.type in('number',G,_F):A=C
		H=B._get_module_context().create_context(A);D=helpers.infer(B._inference_state,H,A);D=convert_values(D,only_stubs=only_stubs,prefer_stubs=prefer_stubs);I=[classes.Name(B._inference_state,A.name)for A in D];return helpers.sorted_definitions(set(I))
	@validate_line_column
	def goto(self,line=_A,column=_A,*,follow_imports=_B,follow_builtin_imports=_B,only_stubs=_B,prefer_stubs=_B):
		G=prefer_stubs;F=only_stubs;E=column;B=self;B._inference_state.reset_recursion_limitations();'\n        Goes to the name that defined the object under the cursor. Optionally\n        you can follow imports.\n        Multiple objects may be returned, depending on an if you can have two\n        different versions of a function.\n\n        :param follow_imports: The method will follow imports.\n        :param follow_builtin_imports: If ``follow_imports`` is True will try\n            to look up names in builtins (i.e. compiled or extension modules).\n        :param only_stubs: Only return stubs for this method.\n        :param prefer_stubs: Prefer stubs to Python objects for this method.\n        :rtype: list of :class:`.Name`\n        ';D=B._module_node.get_name_of_position((line,E))
		if D is _A:return B.infer(line,E,only_stubs=F,prefer_stubs=G)
		C=B._get_module_context().create_name(D);A=[]
		if C.tree_name.is_definition()and C.parent_context.is_class():
			I=C.parent_context.tree_node;J=B._get_module_context().create_value(I);H=J.py__mro__();next(H)
			for K in H:
				A=K.goto(D.value)
				if A:break
		if not A:A=list(C.goto())
		if follow_imports:A=helpers.filter_follow_imports(A,follow_builtin_imports)
		A=convert_names(A,only_stubs=F,prefer_stubs=G);L=[classes.Name(B._inference_state,A)for A in set(A)];return list(set(helpers.sorted_definitions(L)))
	def search(A,string,*,all_scopes=_B):'\n        Searches a name in the current file. For a description of how the\n        search string should look like, please have a look at\n        :meth:`.Project.search`.\n\n        :param bool all_scopes: Default False; searches not only for\n            definitions on the top level of a module level, but also in\n            functions and classes.\n        :yields: :class:`.Name`\n        ';return A._search_func(string,all_scopes=all_scopes)
	@to_list
	def _search_func(self,string,all_scopes=_B,complete=_B,fuzzy=_B):A=self;B=A._names(all_scopes=all_scopes);C,D=helpers.split_search_string(string);return search_in_module(A._inference_state,A._get_module_context(),names=B,wanted_type=C,wanted_names=D,complete=complete,fuzzy=fuzzy)
	def complete_search(A,string,**B):'\n        Like :meth:`.Script.search`, but completes that string. If you want to\n        have all possible definitions in a file you can also provide an empty\n        string.\n\n        :param bool all_scopes: Default False; searches not only for\n            definitions on the top level of a module level, but also in\n            functions and classes.\n        :param fuzzy: Default False. Will return fuzzy completions, which means\n            that e.g. ``ooa`` will match ``foobar``.\n        :yields: :class:`.Completion`\n        ';return A._search_func(string,complete=_C,**B)
	@validate_line_column
	def help(self,line=_A,column=_A):
		"\n        Used to display a help window to users.  Uses :meth:`.Script.goto` and\n        returns additional definitions for keywords and operators.\n\n        Typically you will want to display :meth:`.BaseName.docstring` to the\n        user for all the returned definitions.\n\n        The additional definitions are ``Name(...).type == 'keyword'``.\n        These definitions do not have a lot of value apart from their docstring\n        attribute, which contains the output of Python's :func:`help` function.\n\n        :rtype: list of :class:`.Name`\n        ";D=column;C=line;B=self;B._inference_state.reset_recursion_limitations();F=B.goto(C,D,follow_imports=_C)
		if F:return F
		A=B._module_node.get_leaf_for_position((C,D))
		if A is not _A and A.end_pos==(C,D)and A.type=='newline':
			E=A.get_next_leaf()
			if E is not _A and E.start_pos==A.end_pos:A=E
		if A is not _A and A.type in(_F,_E,'error_leaf'):
			def G():
				if A.value in('(',')','[',']'):
					if A.parent.type=='trailer':return _B
					if A.parent.type=='atom':return _B
				C=B._inference_state.grammar;D=C._pgen_grammar.reserved_syntax_strings.keys();return A.value in D
			if G():H=KeywordName(B._inference_state,A.value);return[classes.Name(B._inference_state,H)]
		return[]
	@validate_line_column
	def get_references(self,line=_A,column=_A,**B):
		"\n        Lists all references of a variable in a project. Since this can be\n        quite hard to do for Jedi, if it is too complicated, Jedi will stop\n        searching.\n\n        :param include_builtins: Default ``True``. If ``False``, checks if a definition\n            is a builtin (e.g. ``sys``) and in that case does not return it.\n        :param scope: Default ``'project'``. If ``'file'``, include references in\n            the current module only.\n        :rtype: list of :class:`.Name`\n        ";E='project';A=self;A._inference_state.reset_recursion_limitations()
		def C(include_builtins=_C,scope=E):
			D='file';B=scope
			if B not in(E,D):raise ValueError('Only the scopes "file" and "project" are allowed')
			F=A._module_node.get_name_of_position((line,column))
			if F is _A:return[]
			G=find_references(A._get_module_context(),F,B==D);C=[classes.Name(A._inference_state,B)for B in G]
			if not include_builtins or B==D:C=[A for A in C if not A.in_builtin_module()]
			return helpers.sorted_definitions(C)
		return C(**B)
	@validate_line_column
	def get_signatures(self,line=_A,column=_A):
		'\n        Return the function object of the call under the cursor.\n\n        E.g. if the cursor is here::\n\n            abs(# <-- cursor is here\n\n        This would return the ``abs`` function. On the other hand::\n\n            abs()# <-- cursor is here\n\n        This would return an empty list..\n\n        :rtype: list of :class:`.Signature`\n        ';A=self;A._inference_state.reset_recursion_limitations();C=line,column;B=helpers.get_signature_details(A._module_node,C)
		if B is _A:return[]
		D=A._get_module_context().create_context(B.bracket_leaf);E=helpers.cache_signatures(A._inference_state,D,B.bracket_leaf,A._code_lines,C);debug.speed('func_call followed');return[classes.Signature(A._inference_state,C,B)for C in E.get_signatures()]
	@validate_line_column
	def get_context(self,line=_A,column=_A):
		'\n        Returns the scope context under the cursor. This basically means the\n        function, class or module where the cursor is at.\n\n        :rtype: :class:`.Name`\n        ';G=column;E=self;F=line,G;A=E._module_node.get_leaf_for_position(F,include_prefixes=_C)
		if A.start_pos>F or A.type=='endmarker':
			H=A.get_previous_leaf()
			if H is not _A:A=H
		I=E._get_module_context();D=tree.search_ancestor(A,_G,_H)
		if D is not _A and D.start_pos<F<=D.children[-1].start_pos:B=I.create_value(D).as_context()
		else:B=I.create_context(A)
		while B.name is _A:B=B.parent_context
		C=classes.Name(E._inference_state,B.name)
		while C.type!='module':
			K=C._name;J=K.tree_name
			if J is not _A:
				L=J.get_definition()
				if L.start_pos[1]<G:break
			C=C.parent()
		return C
	def _analysis(B):
		B._inference_state.is_analysis=_C;B._inference_state.analysis_modules=[B._module_node];F=B._get_module_context()
		try:
			for A in get_executable_nodes(B._module_node):
				C=F.create_context(A)
				if A.type in(_G,_H):tree_name_to_values(B._inference_state,C,A.children[1])
				elif isinstance(A,tree.Import):
					D=set(A.get_defined_names())
					if A.is_nested():D|=set(A[-1]for A in A.get_paths())
					for G in D:imports.infer_import(C,G)
				elif A.type=='expr_stmt':
					H=C.infer_node(A)
					for I in A.children[:-1:2]:unpack_tuple_to_dict(C,H,I)
				else:
					if A.type=='name':E=B._inference_state.infer(C,A)
					else:E=infer_call_of_leaf(C,A)
					try_iter_content(E)
				B._inference_state.reset_recursion_limitations()
			J=[A for A in B._inference_state.analysis if B.path==A.path];return sorted(set(J),key=lambda x:x.line)
		finally:B._inference_state.is_analysis=_B
	def get_names(A,**B):'\n        Returns names defined in the current file.\n\n        :param all_scopes: If True lists the names of all scopes instead of\n            only the module namespace.\n        :param definitions: If True lists the names that have been defined by a\n            class, function or a statement (``a = b`` returns ``a``).\n        :param references: If True lists all the names that are not listed by\n            ``definitions=True``. E.g. ``a = b`` returns ``b``.\n        :rtype: list of :class:`.Name`\n        ';C=A._names(**B);return[classes.Name(A._inference_state,B)for B in C]
	def get_syntax_errors(A):'\n        Lists all syntax errors in the current file.\n\n        :rtype: list of :class:`.SyntaxError`\n        ';return parso_to_jedi_errors(A._inference_state.grammar,A._module_node)
	def _names(A,all_scopes=_B,definitions=_C,references=_B):A._inference_state.reset_recursion_limitations();B=A._get_module_context();C=[B.create_name(A)for A in helpers.get_module_names(A._module_node,all_scopes=all_scopes,definitions=definitions,references=references)];return sorted(C,key=lambda x:x.start_pos)
	def rename(A,line=_A,column=_A,*,new_name):'\n        Renames all references of the variable under the cursor.\n\n        :param new_name: The variable under the cursor will be renamed to this\n            string.\n        :raises: :exc:`.RefactoringError`\n        :rtype: :class:`.Refactoring`\n        ';B=A.get_references(line,column,include_builtins=_B);return refactoring.rename(A._inference_state,B,new_name)
	@validate_line_column
	def extract_variable(self,line,column,*,new_name,until_line=_A,until_column=_A):
		'\n        Moves an expression to a new statement.\n\n        For example if you have the cursor on ``foo`` and provide a\n        ``new_name`` called ``bar``::\n\n            foo = 3.1\n            x = int(foo + 1)\n\n        the code above will become::\n\n            foo = 3.1\n            bar = foo + 1\n            x = int(bar)\n\n        :param new_name: The expression under the cursor will be renamed to\n            this string.\n        :param int until_line: The the selection range ends at this line, when\n            omitted, Jedi will be clever and try to define the range itself.\n        :param int until_column: The the selection range ends at this column, when\n            omitted, Jedi will be clever and try to define the range itself.\n        :raises: :exc:`.RefactoringError`\n        :rtype: :class:`.Refactoring`\n        ';C=until_column;B=self;A=until_line
		if A is _A and C is _A:D=_A
		else:
			if A is _A:A=line
			if C is _A:C=len(B._code_lines[A-1])
			D=A,C
		return extract_variable(B._inference_state,B.path,B._module_node,new_name,(line,column),D)
	@validate_line_column
	def extract_function(self,line,column,*,new_name,until_line=_A,until_column=_A):
		'\n        Moves an expression to a new function.\n\n        For example if you have the cursor on ``foo`` and provide a\n        ``new_name`` called ``bar``::\n\n            global_var = 3\n\n            def x():\n                foo = 3.1\n                x = int(foo + 1 + global_var)\n\n        the code above will become::\n\n            global_var = 3\n\n            def bar(foo):\n                return int(foo + 1 + global_var)\n\n            def x():\n                foo = 3.1\n                x = bar(foo)\n\n        :param new_name: The expression under the cursor will be replaced with\n            a function with this name.\n        :param int until_line: The the selection range ends at this line, when\n            omitted, Jedi will be clever and try to define the range itself.\n        :param int until_column: The the selection range ends at this column, when\n            omitted, Jedi will be clever and try to define the range itself.\n        :raises: :exc:`.RefactoringError`\n        :rtype: :class:`.Refactoring`\n        ';C=until_column;B=self;A=until_line
		if A is _A and C is _A:D=_A
		else:
			if A is _A:A=line
			if C is _A:C=len(B._code_lines[A-1])
			D=A,C
		return extract_function(B._inference_state,B.path,B._get_module_context(),new_name,(line,column),D)
	def inline(A,line=_A,column=_A):'\n        Inlines a variable under the cursor. This is basically the opposite of\n        extracting a variable. For example with the cursor on bar::\n\n            foo = 3.1\n            bar = foo + 1\n            x = int(bar)\n\n        the code above will become::\n\n            foo = 3.1\n            x = int(foo + 1)\n\n        :raises: :exc:`.RefactoringError`\n        :rtype: :class:`.Refactoring`\n        ';B=[A._name for A in A.get_references(line,column,include_builtins=_C)];return refactoring.inline(A._inference_state,B)
class Interpreter(Script):
	'\n    Jedi\'s API for Python REPLs.\n\n    Implements all of the methods that are present in :class:`.Script` as well.\n\n    In addition to completions that normal REPL completion does like\n    ``str.upper``, Jedi also supports code completion based on static code\n    analysis. For example Jedi will complete ``str().upper``.\n\n    >>> from os.path import join\n    >>> namespace = locals()\n    >>> script = Interpreter(\'join("").up\', [namespace])\n    >>> print(script.complete()[0].name)\n    upper\n\n    All keyword arguments are same as the arguments for :class:`.Script`.\n\n    :param str code: Code to parse.\n    :type namespaces: typing.List[dict]\n    :param namespaces: A list of namespace dictionaries such as the one\n        returned by :func:`globals` and :func:`locals`.\n    '
	def __init__(B,code,namespaces,*,project=_A,**E):
		D=project;C=namespaces
		try:C=[dict(A)for A in C]
		except Exception:raise TypeError('namespaces must be a non-empty list of dicts.')
		A=E.get('environment',_A)
		if A is _A:A=InterpreterEnvironment()
		elif not isinstance(A,InterpreterEnvironment):raise TypeError('The environment needs to be an InterpreterEnvironment subclass.')
		if D is _A:D=Project(Path.cwd())
		super().__init__(code,environment=A,project=D,**E);B.namespaces=C;B._inference_state.allow_unsafe_executions=settings.allow_unsafe_interpreter_executions;B._inference_state.do_dynamic_params_search=_B
	@cache.memoize_method
	def _get_module_context(self):
		A=self
		if A.path is _A:B=_A
		else:B=KnownContentFileIO(A.path,A._code)
		C=ModuleValue(A._inference_state,A._module_node,file_io=B,string_names=(_D,),code_lines=A._code_lines);return interpreter.MixedModuleContext(C,A.namespaces)
def preload_module(*B):
	'\n    Preloading modules tells Jedi to load a module now, instead of lazy parsing\n    of modules. This can be useful for IDEs, to control which modules to load\n    on startup.\n\n    :param modules: different module names, list of string.\n    '
	for C in B:A='import %s as x; x.'%C;Script(A).complete(1,len(A))
def set_debug_function(func_cb=debug.print_to_stdout,warnings=_C,notices=_C,speed=_C):"\n    Define a callback debug function to get all the debug messages.\n\n    If you don't specify any arguments, debug messages will be printed to stdout.\n\n    :param func_cb: The callback function for debug messages.\n    ";debug.debug_function=func_cb;debug.enable_warning=warnings;debug.enable_notice=notices;debug.enable_speed=speed